package org.gcube.portlets.widgets.wstaskexecutor.client.rpc;

import java.util.List;

import org.gcube.common.workspacetaskexecutor.shared.TaskParameterType;
import org.gcube.common.workspacetaskexecutor.shared.dataminer.TaskComputation;
import org.gcube.common.workspacetaskexecutor.shared.dataminer.TaskConfiguration;
import org.gcube.common.workspacetaskexecutor.shared.dataminer.TaskExecutionStatus;
import org.gcube.common.workspacetaskexecutor.shared.exception.ItemNotConfiguredException;
import org.gcube.common.workspacetaskexecutor.shared.exception.ItemNotExecutableException;
import org.gcube.portlets.widgets.wstaskexecutor.shared.GcubeScope;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;


/**
 * The Interface GreetingService.
 *
 * @author Francesco Mangiacrapa francesco.mangiacrapa@isti.cnr.it
 * May 4, 2018
 */
@RemoteServiceRelativePath("workspacetaskexecutor")
public interface WsTaskExecutorWidgetService extends RemoteService {

	/**
	 * Gets the list of scopes for logged user.
	 *
	 * @return the list of scopes for logged user
	 * @throws Exception the exception
	 */
	List<GcubeScope> getListOfScopesForLoggedUser() throws Exception;

	/**
	 * Monitor task execution status.
	 *
	 * @param configuration the configuration
	 * @param taskComputation the task computation
	 * @return the task execution status
	 * @throws Exception the exception
	 */
	TaskExecutionStatus monitorTaskExecutionStatus(
		TaskConfiguration configuration, TaskComputation taskComputation) throws Exception;


	/**
	 * Check item task configurations.
	 *
	 * @param itemId the item id
	 * @return the list
	 * @throws ItemNotConfiguredException the item not configured exception
	 * @throws Exception the exception
	 */
	List<TaskConfiguration> checkItemTaskConfigurations(String itemId) throws ItemNotConfiguredException, Exception;


	/**
	 * Gets the availables parameter types.
	 *
	 * @return the availables parameter types
	 * @throws Exception the exception
	 */
	List<TaskParameterType> getAvailableParameterTypes() throws Exception;

	/**
	 * @param itemId
	 * @param taskConfiguration
	 * @param isUpdate
	 * @return
	 * @throws Exception
	 */
	Boolean createTaskConfiguration(
		String itemId, TaskConfiguration taskConfiguration, boolean isUpdate)
		throws Exception;

	/**
	 * @param itemId
	 * @return
	 * @throws Exception
	 */
	List<TaskConfiguration> getItemTaskConfigurations(String itemId)
		throws Exception;

	/**
	 * @param taskConfiguration
	 * @return
	 * @throws ItemNotExecutableException
	 * @throws Exception
	 */
	Boolean removeTaskConfiguration(TaskConfiguration taskConfiguration)
		throws ItemNotExecutableException, Exception;
}
